module LazyAttachable
  extend ActiveSupport::Concern

  class InstanceMethods < Module
    def initialize(name)
      define_method("handle_#{name}_upload") do
        if (send "#{name}_upload").present? && (send "#{name}_upload_size_ok?")
          blob = LazyBlob.from_upload(send "#{name}_upload")

          if (attached = (send "attached_#{name}"))
            attached.lazy_blob = blob
          else
            attachment = LazyAttachment.new(
              name: name,
              record: self,
              lazy_blob: blob
            )
            (send "attached_#{name}=", attachment)
          end
        end
      end

      define_method("#{name}_upload_size_ok?") do
        (send "#{name}_upload").tempfile.size <= 1.megabytes
      end

      define_method("validate_size_of_#{name}_upload") do
        if (send "#{name}_upload").present? && !(send "#{name}_upload_size_ok?")
          errors.add("#{name}_upload", "file too large (limit 1 MB)")
        end
      end

      define_method("attached_#{name}_path") do
        if (send "#{name}_attached?")
          digest = (send "attached_#{name}").digest
          Rails.application.routes.url_helpers.lazy_blob_path(digest)
        else
          ""
        end
      end

      define_method("attached_#{name}_url") do
        if (send "#{name}_attached?")
          digest = (send "attached_#{name}").digest
          Rails.application.routes.url_helpers.lazy_blob_url(digest)
        else
          ""
        end
      end

      define_method("#{name}_attached?") do
        (send "attached_#{name}").present?
      end

      define_method("handle_#{name}_digest") do
        digest = (send "#{name}_digest").presence
        attached = (send "attached_#{name}").presence
        same_as_attached = (attached && digest) && (attached.digest == digest)

        if digest && !same_as_attached
          blob = LazyBlob.find_by(digest: digest)

          if blob
            attachment = LazyAttachment.new(
              lazy_blob: blob,
              name: name,
              digest: digest
            )

            (send "attached_#{name}=", attachment)
          end
        end
      end
    end
  end

  class_methods do
    def has_lazy_attached(name)
      include InstanceMethods.new(name)
      attr_accessor "#{name}_upload"
      attr_accessor "#{name}_digest"

      has_one "attached_#{name}".to_sym,
              -> { where(name: name) },
              class_name: "LazyAttachment",
              as: :record,
              autosave: true,
              dependent: :destroy

      scope "with_attached_#{name}".to_sym, -> {
        includes("attached_#{name}")
      }

      validate "validate_size_of_#{name}_upload".to_sym
      before_validation "handle_#{name}_upload".to_sym
      before_validation "handle_#{name}_digest".to_sym
    end
  end
end
