# frozen_string_literal: true

class LazyBlob < ActiveRecord::Base
  has_many :lazy_attachments, dependent: :restrict_with_error

  validates :filename, presence: true
  validates :content_type, presence: true
  validates :content_length,
            presence: true,
            numericality: { greater_than: 0, less_than: 2.gigabytes }
  validates :digest, presence: true, uniqueness: true
  validates :content, presence: true, length: { maximum: 1.megabyte }

  def self.from_upload(uploaded_file)
    file = File.open(uploaded_file.tempfile.path, 'rb')
    data = file.read
    digest = Digest::SHA256.new.update(data).hexdigest
    blob = find_or_initialize_by(digest: digest)

    if blob.new_record?
      blob.content = data
      blob.content_length = uploaded_file.tempfile.size
      blob.filename = uploaded_file.original_filename
      blob.content_type = uploaded_file.content_type
    end

    blob
  ensure
    file.close
  end

  def data_uri
    "data:#{content_type};base64,#{Base64.encode64(content)}"
  end
end
